import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:osam/osam.dart';

import 'presenter.dart';

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  final store = Store(AppState());
  runApp(
    StoreProvider(
      store: store,
      child: MaterialApp(home: PresenterExample()),
    ),
  );
}

class Body extends StatefulWidget {
  @override
  _BodyState createState() => _BodyState();
}

class _BodyState extends State<Body> {
  var c = 0;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Center(
        child: OsamBuilder<int>(
          stream: context.presenter<ExamplePresenter>().count,
          builder: (ctx, snap) => Text(snap.data.toString()),
        ),
      ),
    );
  }
}

class PresenterExample extends StatefulWidget {
  @override
  _PresenterExampleState createState() => _PresenterExampleState();
}

class _PresenterExampleState extends State<PresenterExample> {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: PresenterProvider(
        presenter: ExamplePresenter(),
        child: Body(),
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.plus_one),
        onPressed: () {
          setState(() {});
        },
      ),
    );
  }
}

class MyHomePage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final presenter = context.presenter<ExamplePresenter>();
    return OsamNavigator<ExamplePresenter>(
      key: const ValueKey('mainN'),
      rootNavigator: true,
      screens: {
        MainRoutes.first: Scaffold(
          appBar: AppBar(),
          body: Container(
            color: Colors.pink,
            child: Center(
              child: RaisedButton(
                child: const Text('to 2'),
                onPressed: () {
                  presenter.push(MainRoutes.second);
                },
              ),
            ),
          ),
        ),
        MainRoutes.second: MultiPresenterProvider(
          key: const ValueKey('multi'),
          providers: [
            PresenterProvider(
              presenter: TabExamplePresenter(),
              key: const ValueKey('tab_p'),
            ),
            PresenterProvider(presenter: ExamplePresenter1(), key: const ValueKey('ex1_p')),
            PresenterProvider(presenter: ExamplePresenter2(), key: const ValueKey('ex2_p')),
          ],
          child: TabPage(),
        )
      },
    );
  }
}

class TabPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final presenterTab = context.presenter<TabExamplePresenter>();
    final presenter1 = context.presenter<ExamplePresenter1>();
    final presenter2 = context.presenter<ExamplePresenter2>();
    return OsamTabNavigator<TabExamplePresenter>(
      key: const ValueKey('mainTabN'),
      tabDecoration: const BoxDecoration(color: Colors.pink),
      tabs: {
        TabRoutes.first: RaisedButton(
          key: const ValueKey('tb1'),
          color: Colors.green,
          child: const Text('1'),
          onPressed: () => presenterTab.changeTab(TabRoutes.first),
        ),
        TabRoutes.second: RaisedButton(
          key: const ValueKey('tb2'),
          color: Colors.green,
          child: const Text('2'),
          onPressed: () => presenterTab.changeTab(TabRoutes.second),
        ),
      },
      navigators: {
        TabRoutes.first: OsamNavigator<ExamplePresenter1>(
          key: const ValueKey('sub_n_1'),
          screens: {
            MainRoutes.first: Scaffold(
              appBar: AppBar(title: const Text('first')),
              body: Center(
                child: RaisedButton(
                  child: const Text('to 2'),
                  onPressed: () {
                    presenter1.push(MainRoutes.second);
                  },
                ),
              ),
              backgroundColor: Colors.cyanAccent,
            ),
            MainRoutes.second: Scaffold(
              appBar: AppBar(
                title: const Text('second'),
              ),
              body: Center(
                child: RaisedButton(
                  child: const Text('back'),
                  onPressed: () {
                    presenter1.pop();
                  },
                ),
              ),
              backgroundColor: Colors.cyan,
            ),
          },
        ),
        TabRoutes.second: OsamNavigator<ExamplePresenter2>(
          key: const ValueKey('sub_n_2'),
          screens: {
            MainRoutes.first: Scaffold(
              backgroundColor: Colors.red,
              appBar: AppBar(title: const Text('first')),
              body: Center(
                child: RaisedButton(
                  child: const Text('to 2'),
                  onPressed: () {
                    presenter2.push(MainRoutes.second);
                  },
                ),
              ),
            ),
            MainRoutes.second: Scaffold(
              backgroundColor: Colors.redAccent,
              appBar: AppBar(title: const Text('second')),
              body: Center(
                child: RaisedButton(
                  child: const Text('back'),
                  onPressed: () {
                    presenter2.pop();
                  },
                ),
              ),
            ),
          },
        ),
      },
    );
  }
}

class MainRoutes {
  static const first = 'first';
  static const second = 'second';
}

class TabRoutes {
  static const first = 'first';
  static const second = 'second';
}

// ignore: must_be_immutable
class AppState extends BaseState<AppState> {
  var count = 0;
  var mainNavigation = NavigationState(routes: [MainRoutes.first]);
  var tabNavigation = TabNavigationState(currentTab: TabRoutes.first);

  var mainNavigation1 = NavigationState(routes: [MainRoutes.first]);
  var mainNavigation2 = NavigationState(routes: [MainRoutes.first]);

  @override
  List<Object> get props =>
      [mainNavigation, tabNavigation, mainNavigation1, mainNavigation2, count];
}
