import 'package:osam/domain/event/event.dart';
import 'package:osam/domain/state/base_state.dart';
import 'package:osam/navigation/domain/navigation_state.dart';
import 'package:osam/presentation/presenter.dart';
import 'package:osam/util/state_stream.dart';

class PresenterWithNavigation<ST extends BaseState<ST>> extends Presenter<ST> {
  NavigationState state;

  StateStream<List<String>> get routeStream =>
      state.propertyStream((state) => state.routes);

  void pop() => store.dispatchEvent(event: Event(reducer: (_) => state..pop()));

  void push(String route) =>
      store.dispatchEvent(event: Event(reducer: (_) => state..push(route)));

  void pushReplacement(String route) => store.dispatchEvent(
      event: Event(reducer: (_) => state..pushReplacement(route)));

  void pushAndRemoveUntil(String route) => store.dispatchEvent(
      event: Event(reducer: (_) => state..pushAndRemoveUntil(route)));
}

class PresenterWithTabNavigation<ST extends BaseState<ST>>
    extends Presenter<ST> {
  TabNavigationState state;

  StateStream<String> get tabStream => StateStream(
      (state.propertyStream((state) => state.currentTab)), state.currentTab);

  void changeTab(String tab) =>
      store.dispatchEvent(event: Event(reducer: (_) => state..changeTab(tab)));
}
