import 'package:flutter/material.dart';
import 'package:osam/util/state_stream.dart';

typedef Builder<T> = Widget Function(BuildContext context, AsyncSnapshot<T> snapshot);
typedef ErrorBuilder = Widget Function(BuildContext context, Object error);

class OsamBuilder<T> extends StatelessWidget {
  final StateStream<T> stream;
  final Builder<T> builder;
  final ErrorBuilder errorBuilder;

  const OsamBuilder({Key key, this.stream, this.builder, this.errorBuilder}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T>(
        key: ValueKey(key.toString() + 'stB'),
        stream: stream.stream,
        initialData: stream.initialData,
        builder: (ctx, snapshot) =>
            snapshot.hasError ? errorBuilder(ctx, snapshot.error) : builder(ctx, snapshot));
  }
}
